from typing import Optional
from typing_extensions import Literal


from pydantic.main import BaseModel


class MaxChanges(BaseModel):
    enabled: bool
    limit: int


class Driving(BaseModel):
    type: Literal['driving'] = 'driving'
    disable_border_crossing: Optional[bool] = None


class Walking(BaseModel):
    type: Literal['walking'] = 'walking'


class Cycling(BaseModel):
    type: Literal['cycling'] = 'cycling'


class Ferry(BaseModel):
    type: Literal['ferry', 'cycling+ferry', 'driving+ferry'] = 'ferry'
    boarding_time: Optional[int] = None


class DrivingTrain(BaseModel):
    type: Literal['driving+train'] = 'driving+train'
    pt_change_delay: Optional[int] = None
    driving_time_to_station: Optional[int] = None
    parking_time: Optional[int] = None
    walking_time: Optional[int] = None
    max_changes: Optional[MaxChanges] = None


class PublicTransport(BaseModel):
    type: Literal[
        'public_transport',
        'train',
        'bus',
        'coach'
    ] = 'public_transport'
    pt_change_delay: Optional[int] = None
    walking_time: Optional[int] = None
    max_changes: Optional[int] = None



import typing
from datetime import datetime

from typing import List, Optional

from pydantic.main import BaseModel

from traveltimepy import Coordinates, Range, PublicTransport, Driving, Ferry, Walking, Cycling, DrivingTrain
from traveltimepy.dto.requests.request import TravelTimeRequest
from traveltimepy.dto.responses.time_map import TimeMapResponse
from traveltimepy.itertools import split, flatten


class DepartureSearch(BaseModel):
    id: str
    coords: Coordinates
    departure_time: datetime
    travel_time: int
    transportation: typing.Union[PublicTransport, Driving, Ferry, Walking, Cycling, DrivingTrain]
    range: Optional[Range] = None


class ArrivalSearch(BaseModel):
    id: str
    coords: Coordinates
    arrival_time: datetime
    travel_time: int
    transportation: typing.Union[PublicTransport, Driving, Ferry, Walking, Cycling, DrivingTrain]
    range: Optional[Range] = None


class Intersection(BaseModel):
    id: str
    search_ids: List[str]


class Union(BaseModel):
    id: str
    search_ids: List[str]


class TimeMapRequest(TravelTimeRequest[TimeMapResponse]):
    departure_searches: List[DepartureSearch]
    arrival_searches: List[ArrivalSearch]
    unions: List[Union]
    intersections: List[Intersection]

    def split_searches(self) -> List[TravelTimeRequest]:
        return [
            TimeMapRequest(
                departure_searches=departures,
                arrival_searches=arrivals,
                unions=self.unions,
                intersections=self.intersections
            )
            for departures, arrivals in split(self.departure_searches, self.arrival_searches, 10)
        ]

    def merge(self, responses: List[TimeMapResponse]) -> TimeMapResponse:
        if len(self.unions) != 0:
            print(responses)
            return TimeMapResponse(
                results=list(filter(
                    lambda res: res.search_id == 'Union search',
                    flatten([response.results for response in responses])
                ))
            )
        elif len(self.intersections) != 0:
            return TimeMapResponse(
                results=list(filter(
                    lambda res: res.search_id == 'Intersection search',
                    flatten([response.results for response in responses])
                ))
            )
        else:
            return TimeMapResponse(
                results=sorted(flatten([response.results for response in responses]), key=lambda res: res.search_id)
            )



import os
from typing import List

import pytest

from traveltimepy import Location, Coordinates
from traveltimepy.sdk import TravelTimeSdk


@pytest.fixture
def sdk() -> TravelTimeSdk:
    return TravelTimeSdk(os.environ['APP_ID'], os.environ['API_KEY'])


@pytest.fixture
def proto_sdk() -> TravelTimeSdk:
    return TravelTimeSdk(os.environ['PROTO_APP_ID'], os.environ['PROTO_API_KEY'])


@pytest.fixture
def locations() -> List[Location]:
    return [
        Location(id='London center', coords=Coordinates(lat=51.508930, lng=-0.131387)),
        Location(id='Hyde Park', coords=Coordinates(lat=51.508824, lng=-0.167093)),
        Location(id='ZSL London Zoo', coords=Coordinates(lat=51.536067, lng=-0.153596))
    ]



from dataclasses import dataclass
from enum import Enum


@dataclass
class TransportationInfo:
    code: int
    name: str


class ProtoTransportation(Enum):
    PUBLIC_TRANSPORT = TransportationInfo(0, 'pt')
    DRIVING = TransportationInfo(1, 'driving')
    DRIVING_FERRY = TransportationInfo(3, 'driving+ferry')
    WALKING = TransportationInfo(4, 'walking')
    CYCLING = TransportationInfo(5, 'cycling')
    CYCLING_FERRY = TransportationInfo(6, 'cycling+ferry')
    WALKING_FERRY = TransportationInfo(7, 'walking+ferry')


class ProtoCountry(str, Enum):
    NETHERLANDS = 'nl'
    AUSTRIA = 'at'
    UNITED_KINGDOM = 'uk'
    BELGIUM = 'be'
    GERMANY = 'de'
    FRANCE = 'fr'
    IRELAND = 'ie'
    LITHUANIA = 'lt'



from datetime import datetime

from traveltimepy import Coordinates, Driving


def test_departures(sdk):
    results = sdk.time_map(
        coordinates=[Coordinates(lat=51.507609, lng=-0.128315), Coordinates(lat=51.517609, lng=-0.138315)],
        departure_time=datetime.now(),
        travel_time=900,
        transportation=Driving()
    )
    assert len(results) == 2


def test_arrivals(sdk):
    results = sdk.time_map(
        coordinates=[Coordinates(lat=51.507609, lng=-0.128315), Coordinates(lat=51.517609, lng=-0.138315)],
        arrival_time=datetime.now(),
        travel_time=900,
        transportation=Driving()
    )
    assert len(results) == 2


def test_union_departures(sdk):
    result = sdk.union(
        coordinates=[Coordinates(lat=51.507609, lng=-0.128315), Coordinates(lat=51.517609, lng=-0.138315)],
        departure_time=datetime.now(),
        travel_time=900,
        transportation=Driving()
    )
    assert len(result.shapes) > 0


def test_intersection_arrivals(sdk):
    result = sdk.intersection(
        coordinates=[Coordinates(lat=51.507609, lng=-0.128315), Coordinates(lat=51.517609, lng=-0.138315)],
        arrival_time=datetime.now(),
        travel_time=900,
        transportation=Driving()
    )
    assert len(result.shapes) > 0



from traveltimepy.dto.requests.time_filter_fast import Transportation


def test_one_to_many(sdk, locations):
    results = sdk.time_filter_fast(
        locations=locations,
        search_ids={
            'London center': ['Hyde Park', 'ZSL London Zoo'],
            'ZSL London Zoo': ['Hyde Park', 'London center'],
        },
        transportation=Transportation(type='public_transport'),
    )

    assert len(results) > 0


def test_many_to_one(sdk, locations):
    results = sdk.time_filter_fast(
        locations=locations,
        search_ids={
            'London center': ['Hyde Park', 'ZSL London Zoo'],
            'ZSL London Zoo': ['Hyde Park', 'London center'],
        },
        transportation=Transportation(type='public_transport'),
        one_to_many=False
    )

    assert len(results) > 0



from typing import Dict, List

from pydantic.main import BaseModel


class ResponseError(BaseModel):
    error_code: int
    description: str
    documentation_link: str
    additional_info: Dict[str, List[str]]



from typing import List, Optional
from pydantic import BaseModel


class Ticket(BaseModel):
    type: str
    price: float
    currency: str


class Fares(BaseModel):
    tickets_total: List[Ticket]


class Properties(BaseModel):
    travel_time: int
    fares: Optional[Fares]


class Location(BaseModel):
    id: str
    properties: Properties


class TimeFilterFastResult(BaseModel):
    search_id: str
    locations: List[Location]
    unreachable: List[str]


class TimeFilterFastResponse(BaseModel):
    results: List[TimeFilterFastResult]



import time
from datetime import datetime

from benchmarks.common import generate_locations
from traveltimepy import TravelTimeSdk, Driving


def generate_matrix(size: int):
    sdk = TravelTimeSdk('APP_ID', 'API_KEY')
    locations = generate_locations(51.507609, -0.128315, 0.05, 'Location', size)
    location_ids = [location.id for location in locations]
    search_ids = [
        (location_id, list(filter(lambda cur_id: cur_id != location_id, location_ids)))
        for location_id in location_ids
    ]
    return sdk.time_filter(
        locations=locations,
        search_ids=dict(search_ids),
        transportation=Driving(),
        arrival_time=datetime.now(),
    )


if __name__ == '__main__':
    start = time.perf_counter()
    response = generate_matrix(50)
    request_time = time.perf_counter() - start
    print("Request completed in {0:.0f}s for matrix 50 * 50".format(request_time))

    start = time.perf_counter()
    response2 = generate_matrix(100)
    request_time = time.perf_counter() - start
    print("Request completed in {0:.0f}s for matrix 100 * 100".format(request_time))

    start = time.perf_counter()
    response3 = generate_matrix(300)
    request_time = time.perf_counter() - start
    print("Request completed in {0:.0f}s for matrix 300 * 300".format(request_time))



"""Python sdk for working with traveltime api"""

from traveltimepy.dto.transportation import PublicTransport, Driving, Ferry, Walking, Cycling, DrivingTrain
from traveltimepy.dto.requests.time_filter_proto import ProtoTransportation, ProtoCountry
from traveltimepy.dto.common import Coordinates, Location, Property, FullRange, Range, Rectangle

from traveltimepy.sdk import TravelTimeSdk
from traveltimepy.dto.requests.time_filter_fast import Transportation
from traveltimepy.dto.requests.zones import ZonesProperty
from traveltimepy.version import __version__

__all__ = [
    '__version__',
    'PublicTransport',
    'Driving',
    'Ferry',
    'Walking',
    'Cycling',
    'DrivingTrain',
    'ProtoTransportation',
    'ProtoCountry',
    'Coordinates',
    'Location',
    'Property',
    'FullRange',
    'Range',
    'Rectangle',
    'TravelTimeSdk',
    'Transportation',
    'ZonesProperty',
]



import itertools
from typing import List, TypeVar, Tuple, Optional

T = TypeVar('T')
R = TypeVar('R')


def sliding(values: List[T], window_size: int) -> List[List[T]]:
    return [values[i:i+window_size] for i in range(0, len(values), window_size)]


def split(left: List[T], right: List[R], window_size: int) -> List[List[Tuple[T, R]]]:
    return list(itertools.zip_longest(sliding(left, window_size), sliding(right, window_size), fillvalue=[]))


def join_opt(values: Optional[List[str]], sep: str) -> Optional[str]:
    return sep.join(values) if values is not None and len(values) != 0 else None


def flatten(list_of_lists: List[List[T]]) -> List[T]:
    return list(itertools.chain.from_iterable(list_of_lists))



import asyncio
from typing import Dict

from aiohttp import ClientSession, ClientResponse, BasicAuth

from traveltimepy.TimeFilterFastResponse_pb2 import TimeFilterFastResponse
from traveltimepy.TimeFilterFastRequest_pb2 import TimeFilterFastRequest
from traveltimepy.dto.responses.time_filter_proto import TimeFilterProtoResponse
from traveltimepy.errors import ApiError


async def send_proto_async(
    url: str,
    headers: Dict[str, str],
    data: TimeFilterFastRequest,
    app_id: str,
    api_key: str
) -> TimeFilterFastResponse:
    async with ClientSession() as session:
        async with session.post(
            url=url,
            headers=headers,
            data=data.SerializeToString(),
            auth=BasicAuth(app_id, api_key)
        ) as resp:
            return await __process_response(resp)


def send_proto(
    url: str,
    headers: Dict[str, str],
    data: TimeFilterFastRequest,
    app_id: str,
    api_key: str
) -> TimeFilterFastResponse:
    return asyncio.run(send_proto_async(url, headers, data, app_id, api_key))


async def __process_response(response: ClientResponse) -> TimeFilterProtoResponse:
    content = await response.read()
    if response.status != 200:
        msg = f'Travel Time API proto request failed with error code: {response.status}\n'
        raise ApiError(msg)
    else:
        response_body = TimeFilterFastResponse()
        response_body.ParseFromString(content)
        return TimeFilterProtoResponse(travel_times=response_body.properties.travelTimes[:])



from datetime import datetime

from traveltimepy import PublicTransport


def test_departures(sdk, locations):
    results = sdk.routes(
        locations=locations,
        search_ids={
            'London center': ['Hyde Park', 'ZSL London Zoo'],
            'ZSL London Zoo': ['Hyde Park', 'London center'],
        },
        transportation=PublicTransport(),
        departure_time=datetime.now()
    )
    assert len(results) == 2


def test_arrivals(sdk, locations):
    results = sdk.routes(
        locations=locations,
        search_ids={
            'London center': ['Hyde Park', 'ZSL London Zoo'],
            'ZSL London Zoo': ['Hyde Park', 'London center'],
        },
        transportation=PublicTransport(),
        departure_time=datetime.now()
    )
    assert len(results) == 2
